==> queueshare-0.0.55.tgz: package/src/qsHttpServerSetup.js <==
import QsYtChannelReleaseResultsOrNull from './QsYtChannelReleaseResultsOrNull.js'
import FdStats from './FdStats.js'
import fs from 'node:fs'
import FsPath from 'node:path'
import FsPathExists from './FsPathExists.js'
import Gzipped from './Gzipped.js'
import httpServeFile from './httpServeFile.js'
import HttpServerReqAsJsonValue from './HttpServerReqAsJsonValue.js'
import IsCollabId from './IsCollabId.js'
import qsBufferHttpServerReqBodyOrNull from './qsBufferHttpServerReqBodyOrNull.js'
import qsBundledQcSetupJs from './qsBundledQcSetupJs.js'
import qsBundledQcSetupJsHash from './qsBundledQcSetupJsHash.js'
import qsHttpServer from './qsHttpServer.js'
import qpsPackage from './qpsPackage.js'
import qsQcCss from './qsQcCss.js'
import qsQcCssHash from './qsQcCssHash.js'
import QsQcHtml from './QsQcHtml.js'
import qsQcWsServer from './qsQcWsServer.js'
import QsRemoteQcUriOrNull from './QsRemoteQcUriOrNull.js'
import QsTrackAudioFileInfoOrNull from './QsTrackAudioFileInfoOrNull.js'
import qsTrackAudioFilesDir from './qsTrackAudioFilesDir.js'
import qsWriteDebuggingInfo from './qsWriteDebuggingInfo.js'
import QsYtAudioUriOrNull from './QsYtAudioUriOrNull.js'
import QsYtChannelRecentVideoResultsOrNull from './QsYtChannelRecentVideoResultsOrNull.js'
import QsYtChannelResultsOrNull from './QsYtChannelResultsOrNull.js'
import QsYtPlaylistVideoResultsOrNull from './QsYtPlaylistVideoResultsOrNull.js'
import QsYtSearchResultsOrNull from './QsYtSearchResultsOrNull.js'
import QsYtVideoRelatedVideoResultsOrNull from './QsYtVideoRelatedVideoResultsOrNull.js'
import QueryString from './QueryString.js'
import ReadFdOrNull from './ReadFdOrNull.js'
import RelativeUriAsUrl from './RelativeUriAsUrl.js'

import './qsHttpServerListen.js'

const writeHead = (res, statusCode, headersOrUndefined) => {
	res.writeHead(statusCode, {'Cache-Control': 'no-store', ...headersOrUndefined})
}
const bundledQcSetupPathname = '/bundledQcSetups/' + encodeURIComponent(qsBundledQcSetupJsHash)
const qcCssPathname = '/qcCss/' + encodeURIComponent(qsQcCssHash)
const [gzippedBundledQcSetupJs, gzippedQcCss] = await Promise.all([Gzipped(qsBundledQcSetupJs), Gzipped(qsQcCss)])
const IsTrackAudioFileName = IsCollabId
qsHttpServer.removeAllListeners('request')
qsHttpServer.on('request', async (req, res) => {
	const url = RelativeUriAsUrl(req.url)
	const pathname = url.pathname
	const searchParams = url.searchParams
	if (pathname === '/') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Encoding': 'gzip', 'Content-Type': 'text/html; charset=utf-8'})
			res.end(await Gzipped(QsQcHtml()))
		} else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	} else if (pathname === bundledQcSetupPathname) {
		if (req.method === 'GET') {
			writeHead(res, 200, {
				'Cache-Control': 'public, max-age=30240000, immutable',
				'Content-Encoding': 'gzip',
				'Content-Type': 'text/javascript',
			})
			res.end(gzippedBundledQcSetupJs)
		} else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	} else if (pathname === '/qcAppleTouchIcon') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Type': 'image/png'})
			httpServeFile(fs.createReadStream(FsPath.join(qpsPackage, 'src', 'qcAppleTouchIcon.png')), res)
		} else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	} else if (pathname === qcCssPathname) {
		if (req.method === 'GET') {
			writeHead(res, 200, {
				'Cache-Control': 'public, max-age=30240000, immutable',
				'Content-Encoding': 'gzip',
				'Content-Type': 'text/css; charset=utf-8',
			})
			res.end(gzippedQcCss)
		} else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	} else if (pathname === '/qcFavicon') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Type': 'image/png'})
			httpServeFile(fs.createReadStream(FsPath.join(qpsPackage, 'src', 'qcFavicon.png')), res)
		} else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	} else if (pathname === '/remoteQcUriOrNull') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Type': 'application/json'})
			res.end(JSON.stringify(QsRemoteQcUriOrNull()))
		} else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	} else if (pathname === '/trackAudioFromFile') {
		if (req.method === 'GET') {
			if (IsTrackAudioFileName(searchParams.get('name'))) {
				const fdOrNull = await ReadFdOrNull(FsPath.join(qsTrackAudioFilesDir, searchParams.get('name')))
				if (fdOrNull === null) {
					writeHead(res, 500)
					res.end()
				} else {
					const contentType = searchParams.get('contentType')
					if (req.headers['range'] === undefined) {
						writeHead(res, 200, {'Content-Type': contentType})
						httpServeFile(fs.createReadStream(null, {fd: fdOrNull}), res)
					} else {
						const rangeMatchOrNull = req.headers['range'].match(/^bytes=(\d{1,13})-$/)
						if (rangeMatchOrNull === null) {
							writeHead(res, 200, {'Content-Type': contentType})
							httpServeFile(fs.createReadStream(null, {fd: fdOrNull}), res)
						} else {
							const [, startAsString] = rangeMatchOrNull
							const start = parseInt(startAsString)
							const size = (await FdStats(fdOrNull)).size
							if (start < size) {
								writeHead(res, 206, {
									'Content-Length': (size - start).toString(),
									'Content-Range': 'bytes ' + start.toString() + '-' + (size - 1).toString() + '/' + size.toString(),
									'Content-Type': contentType,
								})
								httpServeFile(fs.createReadStream(null, {fd: fdOrNull, start: start}), res)
							} else {
								writeHead(res, 200, {'Content-Type': contentType})
								httpServeFile(fs.createReadStream(null, {fd: fdOrNull}), res)
							}
						}
					}
				}
			} else {
				writeHead(res, 500)
				res.end()
			}
		} else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	} else if (pathname === '/writeDebuggingInfo') {
		if (req.method === 'POST') {
			const bodyOrNull = await qsBufferHttpServerReqBodyOrNull(req)
			qsWriteDebuggingInfo(
				'qsHttpServerWriteDebuggingInfoReqBodyOrNull',
				bodyOrNull === null ? null : bodyOrNull.toString('base64'),
			)
			writeHead(res, 204)
			res.end()
		} else {
			writeHead(res, 405, {'Allow': 'POST'})
			res.end()
		}
	} else if (pathname === '/ytChannelRecentVideoResultsOrNull') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Encoding': 'gzip', 'Content-Type': 'application/json'})
			res.end(await Gzipped(JSON.stringify(await QsYtChannelRecentVideoResultsOrNull(
				searchParams.get('channel'),
				searchParams.get('channelTitle'),
			))))
		} else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	} else if (pathname === '/ytChannelResultsOrNull2') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Encoding': 'gzip', 'Content-Type': 'application/json'})
			res.end(await Gzipped(JSON.stringify(await QsYtChannelResultsOrNull(
				searchParams.get('channel'),
				searchParams.get('channelTitle'),
			))))
		} else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	} else if (pathname === '/ytPlaylistVideoResultsOrNull') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Encoding': 'gzip', 'Content-Type': 'application/json'})
			res.end(await Gzipped(JSON.stringify(await QsYtPlaylistVideoResultsOrNull(searchParams.get('playlist')))))
		} else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	} else if (pathname === '/ytSearchResultsOrNull') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Encoding': 'gzip', 'Content-Type': 'application/json'})
			res.end(await Gzipped(JSON.stringify(await QsYtSearchResultsOrNull(searchParams.get('query')))))
		} else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	} else if (pathname === '/ytTrackAudio2') {
		if (req.method === 'GET') {
			const fileInfoOrNull = QsTrackAudioFileInfoOrNull(searchParams.get('track'))
			if (
				fileInfoOrNull !== null
				&& IsTrackAudioFileName(fileInfoOrNull.name)
				&& await FsPathExists(FsPath.join(qsTrackAudioFilesDir, fileInfoOrNull.name))
			) {
				writeHead(res, 307, {
					'Location': '/trackAudioFromFile?' + QueryString({
						contentType: fileInfoOrNull.contentType,
						name: fileInfoOrNull.name,
					}),
				})
				res.end()
			} else {
				const uriOrNull = await QsYtAudioUriOrNull(searchParams.get('video'))
				if (uriOrNull === null) {
					writeHead(res, 500)
					res.end()
				} else {
					writeHead(res, 307, {'Location': uriOrNull})
					res.end()
				}
			}
		} else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	} else if (pathname === '/ytVideoRelatedVideoResultsOrNull') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Encoding': 'gzip', 'Content-Type': 'application/json'})
			res.end(await Gzipped(JSON.stringify(await QsYtVideoRelatedVideoResultsOrNull(searchParams.get('video')))))
		} else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	} else if (pathname === '/ytChannelReleaseResultsOrNull') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Encoding': 'gzip', 'Content-Type': 'application/json'})
			res.end(await Gzipped(JSON.stringify(await QsYtChannelReleaseResultsOrNull(
				searchParams.get('channel'),
				searchParams.get('channelTitle'),
			))))
		} else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	} else {
		writeHead(res, 404)
		res.end()
	}
	qsWriteDebuggingInfo('qsHttpServerReq', HttpServerReqAsJsonValue(req))
})
qsHttpServer.on('upgrade', (req, socket, head) => {
	const pathname = RelativeUriAsUrl(req.url).pathname
	if (pathname === '/qcWs') {
		if (req.method === 'GET') {
			qsQcWsServer.handleUpgrade(req, socket, head, (ws) => {
				qsQcWsServer.emit('connection', ws, req)
			})
		} else {
			socket.destroy()
		}
	} else {
		socket.destroy()
	}
	qsWriteDebuggingInfo('qsHttpServerUpgradeReq', HttpServerReqAsJsonValue(req))
})

==> queueshare-0.0.57.tgz: package/src/qsHttpServerSetup.js <==
import QsYtChannelTabResultsOrNull from './QsYtChannelTabResultsOrNull.js'
import FdStats from './FdStats.js'
import fs from 'node:fs'
import FsPath from 'node:path'
import FsPathExists from './FsPathExists.js'
import Gzipped from './Gzipped.js'
import httpServeFile from './httpServeFile.js'
import HttpServerReqAsJsonValue from './HttpServerReqAsJsonValue.js'
import IsCollabId from './IsCollabId.js'
import qsBufferHttpServerReqBodyOrNull from './qsBufferHttpServerReqBodyOrNull.js'
import qsBundledQcSetupJs from './qsBundledQcSetupJs.js'
import qsBundledQcSetupJsHash from './qsBundledQcSetupJsHash.js'
import qsHttpServer from './qsHttpServer.js'
import qpsPackage from './qpsPackage.js'
import qsQcCss from './qsQcCss.js'
import qsQcCssHash from './qsQcCssHash.js'
import QsQcHtml from './QsQcHtml.js'
import qsQcWsServer from './qsQcWsServer.js'
import QsRemoteQcUriOrNull from './QsRemoteQcUriOrNull.js'
import QsTrackAudioFileInfoOrNull from './QsTrackAudioFileInfoOrNull.js'
import qsTrackAudioFilesDir from './qsTrackAudioFilesDir.js'
import qsWriteDebuggingInfo from './qsWriteDebuggingInfo.js'
import QsYtAudioUriOrNull from './QsYtAudioUriOrNull.js'
import QsYtChannelResultsOrNull from './QsYtChannelResultsOrNull.js'
import QsYtPlaylistVideoResultsOrNull from './QsYtPlaylistVideoResultsOrNull.js'
import QsYtSearchResultsOrNull from './QsYtSearchResultsOrNull.js'
import QsYtVideoRelatedVideoResultsOrNull from './QsYtVideoRelatedVideoResultsOrNull.js'
import QueryString from './QueryString.js'
import ReadFdOrNull from './ReadFdOrNull.js'
import RelativeUriAsUrl from './RelativeUriAsUrl.js'

import './qsHttpServerListen.js'

const writeHead = (res, statusCode, headersOrUndefined) => {
	res.writeHead(statusCode, {'Cache-Control': 'no-store', ...headersOrUndefined})
}
const bundledQcSetupPathname = '/bundledQcSetups/' + encodeURIComponent(qsBundledQcSetupJsHash)
const qcCssPathname = '/qcCss/' + encodeURIComponent(qsQcCssHash)
const [gzippedBundledQcSetupJs, gzippedQcCss] = await Promise.all([Gzipped(qsBundledQcSetupJs), Gzipped(qsQcCss)])
const IsTrackAudioFileName = IsCollabId
qsHttpServer.removeAllListeners('request')
qsHttpServer.on('request', async (req, res) => {
	const url = RelativeUriAsUrl(req.url)
	const pathname = url.pathname
	const searchParams = url.searchParams
	if (pathname === '/') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Encoding': 'gzip', 'Content-Type': 'text/html; charset=utf-8'})
			res.end(await Gzipped(QsQcHtml()))
		}
		else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	}
	else if (pathname === bundledQcSetupPathname) {
		if (req.method === 'GET') {
			writeHead(res, 200, {
				'Cache-Control': 'public, max-age=30240000, immutable',
				'Content-Encoding': 'gzip',
				'Content-Type': 'text/javascript',
			})
			res.end(gzippedBundledQcSetupJs)
		}
		else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	}
	else if (pathname === '/qcAppleTouchIcon') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Type': 'image/png'})
			httpServeFile(fs.createReadStream(FsPath.join(qpsPackage, 'src', 'qcAppleTouchIcon.png')), res)
		}
		else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	}
	else if (pathname === qcCssPathname) {
		if (req.method === 'GET') {
			writeHead(res, 200, {
				'Cache-Control': 'public, max-age=30240000, immutable',
				'Content-Encoding': 'gzip',
				'Content-Type': 'text/css; charset=utf-8',
			})
			res.end(gzippedQcCss)
		}
		else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	}
	else if (pathname === '/qcFavicon') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Type': 'image/png'})
			httpServeFile(fs.createReadStream(FsPath.join(qpsPackage, 'src', 'qcFavicon.png')), res)
		}
		else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	}
	else if (pathname === '/remoteQcUriOrNull') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Type': 'application/json'})
			res.end(JSON.stringify(QsRemoteQcUriOrNull()))
		}
		else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	}
	else if (pathname === '/trackAudioFromFile') {
		if (req.method === 'GET') {
			if (IsTrackAudioFileName(searchParams.get('name'))) {
				const fdOrNull = await ReadFdOrNull(FsPath.join(qsTrackAudioFilesDir, searchParams.get('name')))
				if (fdOrNull === null) {
					writeHead(res, 500)
					res.end()
				}
				else {
					const contentType = searchParams.get('contentType')
					if (req.headers['range'] === undefined) {
						writeHead(res, 200, {'Content-Type': contentType})
						httpServeFile(fs.createReadStream(null, {fd: fdOrNull}), res)
					}
					else {
						const rangeMatchOrNull = req.headers['range'].match(/^bytes=(\d{1,13})-$/)
						if (rangeMatchOrNull === null) {
							writeHead(res, 200, {'Content-Type': contentType})
							httpServeFile(fs.createReadStream(null, {fd: fdOrNull}), res)
						}
						else {
							const [, startAsString] = rangeMatchOrNull
							const start = parseInt(startAsString)
							const size = (await FdStats(fdOrNull)).size
							if (start < size) {
								writeHead(res, 206, {
									'Content-Length': (size - start).toString(),
									'Content-Range': 'bytes ' + start.toString() + '-' + (size - 1).toString() + '/' + size.toString(),
									'Content-Type': contentType,
								})
								httpServeFile(fs.createReadStream(null, {fd: fdOrNull, start: start}), res)
							}
							else {
								writeHead(res, 200, {'Content-Type': contentType})
								httpServeFile(fs.createReadStream(null, {fd: fdOrNull}), res)
							}
						}
					}
				}
			}
			else {
				writeHead(res, 500)
				res.end()
			}
		}
		else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	}
	else if (pathname === '/writeDebuggingInfo') {
		if (req.method === 'POST') {
			const bodyOrNull = await qsBufferHttpServerReqBodyOrNull(req)
			qsWriteDebuggingInfo(
				'qsHttpServerWriteDebuggingInfoReqBodyOrNull',
				bodyOrNull === null ? null : bodyOrNull.toString('base64'),
			)
			writeHead(res, 204)
			res.end()
		}
		else {
			writeHead(res, 405, {'Allow': 'POST'})
			res.end()
		}
	}
	else if (pathname === '/ytChannelRecentVideoResultsOrNull') {
		writeHead(res, 404)
		res.end()
	}
	else if (pathname === '/ytChannelResultsOrNull3') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Encoding': 'gzip', 'Content-Type': 'application/json'})
			res.end(await Gzipped(JSON.stringify(await QsYtChannelResultsOrNull(
				searchParams.get('channel'),
				searchParams.get('channelTitle'),
			))))
		}
		else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	}
	else if (pathname === '/ytPlaylistVideoResultsOrNull') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Encoding': 'gzip', 'Content-Type': 'application/json'})
			res.end(await Gzipped(JSON.stringify(await QsYtPlaylistVideoResultsOrNull(searchParams.get('playlist')))))
		}
		else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	}
	else if (pathname === '/ytSearchResultsOrNull') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Encoding': 'gzip', 'Content-Type': 'application/json'})
			res.end(await Gzipped(JSON.stringify(await QsYtSearchResultsOrNull(searchParams.get('query')))))
		}
		else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	}
	else if (pathname === '/ytTrackAudio2') {
		if (req.method === 'GET') {
			const fileInfoOrNull = QsTrackAudioFileInfoOrNull(searchParams.get('track'))
			if (
				fileInfoOrNull !== null
				&& IsTrackAudioFileName(fileInfoOrNull.name)
				&& await FsPathExists(FsPath.join(qsTrackAudioFilesDir, fileInfoOrNull.name))
			) {
				writeHead(res, 307, {
					'Location': '/trackAudioFromFile?' + QueryString({
						contentType: fileInfoOrNull.contentType,
						name: fileInfoOrNull.name,
					}),
				})
				res.end()
			}
			else {
				const uriOrNull = await QsYtAudioUriOrNull(searchParams.get('video'))
				if (uriOrNull === null) {
					writeHead(res, 500)
					res.end()
				}
				else {
					writeHead(res, 307, {'Location': uriOrNull})
					res.end()
				}
			}
		}
		else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	}
	else if (pathname === '/ytVideoRelatedVideoResultsOrNull') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Encoding': 'gzip', 'Content-Type': 'application/json'})
			res.end(await Gzipped(JSON.stringify(await QsYtVideoRelatedVideoResultsOrNull(searchParams.get('video')))))
		}
		else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	}
	else if (pathname === '/ytChannelReleaseResultsOrNull') {
		writeHead(res, 404)
		res.end()
	}
	else if (pathname === '/ytChannelTabResultsOrNull') {
		if (req.method === 'GET') {
			writeHead(res, 200, {'Content-Encoding': 'gzip', 'Content-Type': 'application/json'})
			res.end(await Gzipped(JSON.stringify(await QsYtChannelTabResultsOrNull(
				searchParams.get('channel'),
				searchParams.get('channelTitle'),
				searchParams.get('tabName'),
			))))
		}
		else {
			writeHead(res, 405, {'Allow': 'GET'})
			res.end()
		}
	}
	else {
		writeHead(res, 404)
		res.end()
	}
	qsWriteDebuggingInfo('qsHttpServerReq', HttpServerReqAsJsonValue(req))
})
qsHttpServer.on('upgrade', (req, socket, head) => {
	const pathname = RelativeUriAsUrl(req.url).pathname
	if (pathname === '/qcWs') {
		if (req.method === 'GET') {
			qsQcWsServer.handleUpgrade(req, socket, head, (ws) => {
				qsQcWsServer.emit('connection', ws, req)
			})
		}
		else {
			socket.destroy()
		}
	}
	else {
		socket.destroy()
	}
	qsWriteDebuggingInfo('qsHttpServerUpgradeReq', HttpServerReqAsJsonValue(req))
})
